// hlavickove subory *******************************************************
// systemove funkcie
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
// GL a GLU kniznica
#include <GL/gl.h>
#include <GL/glu.h>
// funkcie SDL kniznice
#include <SDL/SDL.h>

// vlastne funkcie pre pracu s Frustum
#include "glFrustum.h"

// .h subor
#include "OGL.h"

// instancia triedy CBMPImage
CBMPImage m_cBMPImage;

// instancia triedy FrustumCulling
FrustumCulling m_FrustumCulling;

// konstruktor *************************************************************
COGL::COGL()
{
 rot_x=1.0f; rot_y=1.0f;

 normals_onoff=0;
 octtree_onoff=1;
 cam_mode=0;
 help_onoff=0;
}

// destruktor **************************************************************
COGL::~COGL()
{

}

// inicializacia OpenGL - nastavenie parametrov pre tvorbu OpenGL sceny ****
bool COGL::gl_initialize(void)
{
 // definicia premennych ***************************************************
 for (t_a=0;t_a<8;t_a++) { // nastavenie 8-mich vseobecnych materialov
  ogl_material[t_a].amb[0]=1.0f; ogl_material[t_a].amb[1]=1.0f;
  ogl_material[t_a].amb[2]=1.0f; ogl_material[t_a].amb[3]=1.0f;
  ogl_material[t_a].dif[0]=0.6f; ogl_material[t_a].dif[1]=0.6f;
  ogl_material[t_a].dif[2]=0.6f; ogl_material[t_a].dif[3]=1.0f;
  ogl_material[t_a].spe[0]=1.0f; ogl_material[t_a].spe[1]=1.0f;
  ogl_material[t_a].spe[2]=1.0f; ogl_material[t_a].spe[3]=1.0f;
  ogl_material[t_a].amb_dif[0]=1.0f; ogl_material[t_a].amb_dif[1]=1.0f;
  ogl_material[t_a].amb_dif[2]=1.0f; ogl_material[t_a].amb_dif[3]=1.0f;
  ogl_material[t_a].emi[0]=0.0f; ogl_material[t_a].emi[1]=0.0f;
  ogl_material[t_a].emi[2]=0.0f; ogl_material[t_a].emi[3]=1.0f;
  ogl_material[t_a].shi=0.0f;
  ogl_material[t_a].transp=0.0f;
  ogl_material[t_a].sillum[0]=0.0f; ogl_material[t_a].sillum[1]=0.0f;
  ogl_material[t_a].sillum[2]=0.0f; ogl_material[t_a].sillum[3]=1.0f;
 }

 for (t_a=0;t_a<8;t_a++) { // nastavenie 8-mich vseobecnych svetiel
  ogl_light[t_a].amb[0]=0.0f; ogl_light[t_a].amb[1]=0.0f;
  ogl_light[t_a].amb[2]=0.0f; ogl_light[t_a].amb[3]=1.0f;
  ogl_light[t_a].dif[0]=1.0f; ogl_light[t_a].dif[1]=1.0f;
  ogl_light[t_a].dif[2]=1.0f; ogl_light[t_a].dif[3]=1.0f;
  ogl_light[t_a].spe[0]=1.0f; ogl_light[t_a].spe[1]=1.0f;
  ogl_light[t_a].spe[2]=1.0f; ogl_light[t_a].spe[3]=1.0f;
  ogl_light[t_a].pos[0]=400.0f; ogl_light[t_a].pos[1]=1000.0f;
  ogl_light[t_a].pos[2]=-400.0f; ogl_light[t_a].pos[3]=1.0f;
  ogl_light[t_a].spot[0]=0.0f;
  ogl_light[t_a].spot[1]=0.0f;
  ogl_light[t_a].spot[2]=-1.0f;
 }

 // nastavenie OpenGL renderovacich parametrov *****************************
 glClearColor(0.0f,0.0f,0.0f,0.5f); // zasobnik farieb bude vymazavany ciernou farbou
 glClearDepth(1.0f);                // zasobnik hlbky bude vymazavany nastavenim na 1.0
 // nastavenie parametrov 1 svetla
 glLightfv(GL_LIGHT0,GL_AMBIENT,ogl_light[0].amb);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,ogl_light[0].dif);
 glLightfv(GL_LIGHT0,GL_SPECULAR,ogl_light[0].spe);
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_light[0].pos);
 // zapnutie/vypnutie osvetlenia
 ogl_lighting=1;
 if (ogl_lighting==1) glEnable(GL_LIGHTING);
 else glDisable(GL_LIGHTING);
 glEnable(GL_LIGHT0);

 glDepthFunc(GL_LEQUAL);            // typ porovnavacej funkcie pre urcenie z-vzdialenosti
 glEnable(GL_DEPTH_TEST);           // zapnutie testovania hlbky

 glDisable(GL_CULL_FACE);           // nastavenie poradia, v ktorom budu definovane body objektu
 glFrontFace(GL_CCW);               // reprezentovat privratene resp. odvratene plochy,
 glCullFace(GL_BACK);
 glEnable(GL_CULL_FACE);            // odvratene plochy sa nebudu spracovavat
                                    // vseobecny material objektov
 glMaterialfv(GL_FRONT,GL_AMBIENT_AND_DIFFUSE,ogl_material[0].amb_dif);
                                    // svetelne vypocty su lokalne
 glLightModelf(GL_LIGHT_MODEL_LOCAL_VIEWER,GL_TRUE);
 glShadeModel(GL_SMOOTH);           // zapnutie gourand-jemneho tienovania
                                    // osvetlenie textur alias modulate
 //glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
                                    // realnejsie perspektivne vypocty a korekcie
 glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

 ogl_fogColor[0]=0.0f; ogl_fogColor[1]=0.0f;
 ogl_fogColor[2]=0.0f; ogl_fogColor[3]=1.0f;
 glFogi(GL_FOG_MODE,GL_LINEAR);          // typ hmly
 glFogfv(GL_FOG_COLOR,ogl_fogColor);     // farba hmly
 glFogf(GL_FOG_DENSITY,0.01f);           // hustota hmly
 glHint(GL_FOG_HINT,GL_DONT_CARE);       // dalsie nastavenia
 glFogf(GL_FOG_START,1000.0f);           // pociatok viditelnosti hmly
 glFogf(GL_FOG_END,1500.0f);             // vzdialenost max. hmly
 //glEnable(GL_FOG);									// povolenie hmly

 //glEnableClientState(GL_VERTEX_ARRAY);
 //glEnableClientState(GL_NORMAL_ARRAY);

 //glEnable(GL_LINE_SMOOTH);          // antialiasing ciar alias smoothline
 //glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);
                                    // funkcia pre vypocet priesvitnosti
 //glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
 //glEnable(GL_BL1END);              // zapnutie vypoctu priesvitnosti
 //glLineWidth(1.5);

 //glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

 // inicializacia vypisovania OpenGL textu
 if (ogl_text.text_initialize()) return(1);

 // nahratie textur
 //if (ogl_texture.LoadTexture("data/crate.bmp")) {
 // fprintf(stderr,"Chyba nahratia textur!\n");
 // return(1);
 //}

 char fname[128];
 // nahratie bitmapy terenu
 strcpy(fname,"data/");
 strcat(fname,"terrain");
 strcat(fname,"/");
 strcat(fname,"map1_height");
 strcat(fname,".bmp\0");
 if (!m_cBMPImage.BMPImage_load(fname)) return(1);

 if (ogl_terrain.terrain_create(m_cBMPImage.get_data())) {
  fprintf(stderr,"Chyba pripravy terenu!\n");
  return(1);
 }

 if (m_OctTree.Otree_create(ogl_terrain.terrain_getTriangles(),
                            ogl_terrain.all_triangles)) {
  fprintf(stderr,"Chyba pripravy oktanoveho stromu!\n");
  return(1);
 }

 glEnable(GL_TEXTURE_2D);

 return(0);
}

// vykreslenie OpenGL sceny ************************************************
GLubyte COGL::gl_draw(void)
{
 // vymazanie zasobnika farieb a hlbky
 glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

 // zresetovanie modelovej matice
 glLoadIdentity();

 // TRANSFORMACIA SCENY PODLA NASTAVENI AKTUALNEJ KAMERY!
 glMultMatrixf(ogl_camera[0].matrix);

 glTranslatef(ogl_camera[0].posx,ogl_camera[0].posy,ogl_camera[0].posz);

 // transformacia polohy svetla
 //ogl_light[0].pos[0]=-ogl_camera[0].posx; ogl_light[0].pos[2]=-ogl_camera[0].posz;
 glLightfv(GL_LIGHT0,GL_POSITION,ogl_light[0].pos);

 // ulozenie aktualnej matice do zasobnika matic
 glPushMatrix();

 // kreslim bud z dat terenu alebo z oktanoveho stromu
 /*// kreslenie z dat terenu, FrustumCulling odstranuje trojuholniky
 m_FrustumCulling.FrustumUpdate();
 m_FrustumCulling.CullTriangles(ogl_terrain.terrain_getTriangles(),
                                ogl_terrain.terrain_getTriangles1(),
                                ogl_terrain.terrain_getNormals(),
                                ogl_terrain.terrain_getNormals1(),
                                ogl_terrain.terrain_getTexCoords(),
                                ogl_terrain.terrain_getTexCoords1(),
                                ogl_terrain.all_triangles,
                                &ogl_terrain.cull_triangles);

 if (cam_mode==1) {
  glLoadIdentity();
  glRotatef(90.0f,1.0f,0.0f,0.0f);
  glTranslatef(-500.0f,-1000.0f,500.0f);
  // transformacia polohy svetla
  //ogl_light[0].pos[0]=-ogl_camera[0].posx; ogl_light[0].pos[2]=-ogl_camera[0].posz;
  glLightfv(GL_LIGHT0,GL_POSITION,ogl_light[0].pos);
 }

 ogl_terrain.terrain_render(ogl_terrain.cull_triangles);
 */
 // kreslenie z oktanoveho stromu, FrustumCulling odstranuje listy stromu
 m_OctTree.Otree_render();
 //if (normals_onoff) ogl_terrain.normals_render(ogl_terrain.cull_triangles);

 glPopMatrix();

 glLoadIdentity();
  glRotatef(45.0f,1.0f,0.0f,0.0f);
  glTranslatef(-500.0f,-800.0f,-400.0f);

 // ulozenie aktualnej matice do zasobnika matic
 glPushMatrix();

 if (octtree_onoff) m_OctTree.Otree_draw();

 glPopMatrix();

 // ulozenie aktualnej matice do zasobnika matic
 glPushMatrix();

 glTranslatef(-ogl_camera[0].posx,-ogl_camera[0].posy,-ogl_camera[0].posz);
 //ogl_object.obj_cube1(10);

 glPopMatrix();

 rot_x+=0.5f; rot_y+=0.5f;

 if (help_onoff) {
  // vypis textu - help
  sprintf(ogl_text.mybuf,"ESC        ukoncenie aplikacie");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-20.0f);
  sprintf(ogl_text.mybuf,"SIPKY      posun dopredu,dozadu,vlavo,vpravo");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-40.0f);
  sprintf(ogl_text.mybuf,"A,Z        posun hore,dole");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-50.0f);
  sprintf(ogl_text.mybuf,"SHIFT      zrychlenie posunu");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-60.0f);
  sprintf(ogl_text.mybuf,"MYS        rotacia v danom smere");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-70.0f);
  sprintf(ogl_text.mybuf,"Q,W        rotacia okolo doprednej osi vlavo,vpravo");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-80.0f);
  sprintf(ogl_text.mybuf,"C          zmena pohladu kamery");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-90.0f);
  sprintf(ogl_text.mybuf,"R          reinicializacia pohladu kamery");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-100.0f);
  sprintf(ogl_text.mybuf,"M          vyplnene trojuholniky/iba hrany trojuholnikov");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-110.0f);
  sprintf(ogl_text.mybuf,"F          zmena typu hmly");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-120.0f);
  sprintf(ogl_text.mybuf,"T          zapnutie/vypnutie texturovania");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-130.0f);
  sprintf(ogl_text.mybuf,"N          zapnutie/vypnutie zobrazenia normal");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-140.0f);
  sprintf(ogl_text.mybuf,"O          zapnutie/vypnutie zobrazenia oktanoveho stromu");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-150.0f);
  sprintf(ogl_text.mybuf,"H          zapnutie/vypnutie zobrazenia helpu");
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-170.0f);
 }
 else {
  // vypis textu - poloha kamery
  sprintf(ogl_text.mybuf,"x: %6.2f",-ogl_camera[0].posx);
  ogl_text.DrawString(10.0f,30.0f);
  sprintf(ogl_text.mybuf,"y: %6.2f",-ogl_camera[0].posy);
  ogl_text.DrawString(10.0f,20.0f);
  sprintf(ogl_text.mybuf,"z: %6.2f",-ogl_camera[0].posz);
  ogl_text.DrawString(10.0f,10.0f);

  // vypis textu - pocet obrazkov za sekundu
  sprintf(ogl_text.mybuf,"%i fps",(GLuint) ogl_camera[0].ogl_fps);
  ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-20.0f);

  // vypis textu - pocet vsetkych a orezanych trojuholnikov
  //sprintf(ogl_text.mybuf,"vsetkych trojuholnikov: %i",(GLuint) ogl_terrain.all_triangles);
  //ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-60.0f);
  //sprintf(ogl_text.mybuf,"trojuholnikov v scene: %i",(GLuint) ogl_terrain.cull_triangles);
  //ogl_text.DrawString(10.0f,ogl_camera[0].scr_h-80.0f);
 }

 return(0);
}

// zmena sposobu zobrazenia objektov ***************************************
void COGL::change_PolygonMode(void)
{
 glGetIntegerv(GL_POLYGON_MODE,&ogl_state[0]);

 // vypis nacitaneho stavu
 //fprintf(stderr,"%d\n",ogl_state[0]);

 if (ogl_state[0]==GL_LINE) glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
 else glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);

 SDL_Delay(100);
}

// zmena parametrov hmly ***************************************************
void COGL::change_FogMode(void)
{
 glGetIntegerv(GL_FOG_MODE,&ogl_state[0]);

 // vypis nacitaneho stavu
 //fprintf(stderr,"%d\n",ogl_state[0]);

 if (ogl_state[0]==GL_EXP) glFogi(GL_FOG_MODE,GL_EXP2);
 else if (ogl_state[0]==GL_EXP2) glFogi(GL_FOG_MODE,GL_LINEAR);
      else glFogi(GL_FOG_MODE,GL_EXP);

 SDL_Delay(100);
}

// zap/vyp texturovania ****************************************************
void COGL::change_TexOnOff(void)
{
 if (glIsEnabled(GL_TEXTURE_2D)) glDisable(GL_TEXTURE_2D);
 else glEnable(GL_TEXTURE_2D);

 SDL_Delay(100);
}

// zmena pohladu ***********************************************************
void COGL::change_CamMode(void)
{
 if (cam_mode==0) { cam_mode=1; glDisable(GL_FOG); }
 else { cam_mode=0; glEnable(GL_FOG); }

 SDL_Delay(100);
}

// zap/vyp vykreslovania normal ********************************************
void COGL::change_NormalsOnOff(void)
{
 if (normals_onoff==0) normals_onoff=1; else normals_onoff=0;

 SDL_Delay(100);
}

// zap/vyp vykreslovania oktanoveho stromu *********************************
void COGL::change_OctTreeOnOff(void)
{
 if (octtree_onoff==0) octtree_onoff=1; else octtree_onoff=0;

 SDL_Delay(100);
}

// zap/vyp vypisu helpu ****************************************************
void COGL::change_HelpOnOff(void)
{
 if (help_onoff==0) help_onoff=1; else help_onoff=0;

 SDL_Delay(100);
}

